@@ -18,16 +18,17 @@ import logging
 import math
 import sys
 
-__version__ = "0.14.0"
+__version__ = "0.15.1"
 
 
 log = logging.getLogger(__name__)
 log.addHandler(logging.NullHandler())
+log.addHandler(logging.StreamHandler(sys.stdout))
 
 _eps = sys.float_info.epsilon
 
 
-try:  # pragma: no cover
+try:  # pragma: no cover  # noqa
     import matplotlib.pyplot as plt
 
     def get_default_label(count, circle):
@@ -60,7 +61,10 @@ try:  # pragma: no cover
         ax.set_title(title)
         if lim is None:
             lim = max(
-                max(abs(circle.x) + circle.r, abs(circle.y) + circle.r,)
+                max(
+                    abs(circle.x) + circle.r,
+                    abs(circle.y) + circle.r,
+                )
                 for circle in circles
             )
         plt.xlim(-lim, lim)
@@ -72,13 +76,12 @@ try:  # pragma: no cover
             labels = [get_default_label(i, c) for i, c in enumerate(circles)]
         return _bubbles([c.circle for c in circles], labels, lim)
 
-
 except ImportError:  # pragma: no cover
     pass
 
 
 _Circle = collections.namedtuple("_Circle", ["x", "y", "r"])
-FieldNames = collections.namedtuple("Field", ["id", "datum", "children"])
+Fields = collections.namedtuple("Fields", ["id", "datum", "children"])
 
 
 class Circle:
@@ -182,9 +185,7 @@ def get_intersection(circle1, circle2):
                 circle1,
                 circle2,
             )
-        if math.isclose(d, 0, abs_tol=eps) and math.isclose(
-            r1, r2, rel_tol=0.0, abs_tol=eps
-        ):
+        if math.isclose(d, 0, abs_tol=eps) and math.isclose(r1, r2, rel_tol=0.0, abs_tol=eps):
             log.debug("no solution, circles are coincident: %s, %s", circle1, circle2)
         return None, None
     xm = x1 + a * dx / d
@@ -298,7 +299,7 @@ def place_new_A1_0(radius, next_, const_placed_circles, get_hole_degree):
         return placed_circles
     mhd = None
     lead_candidate = None
-    for (c1, c2) in itertools.combinations(placed_circles, 2):
+    for c1, c2 in itertools.combinations(placed_circles, 2):
         margin = radius * _eps * 10.0
         # Placed circles other than the 2 circles used to find the
         # candidate placement.
@@ -322,7 +323,7 @@ def place_new_A1_0(radius, next_, const_placed_circles, get_hole_degree):
                 break
     if lead_candidate is None:
         # The radius is set to sqrt(value) in pack_A1_0
-        raise ValueError("cannot place circle for value " + str(radius ** 2))
+        raise ValueError("cannot place circle for value " + str(radius**2))
     placed_circles.append(lead_candidate)
     return placed_circles
 
@@ -342,21 +343,19 @@ def pack_A1_0(data):
     min_max_ratio = min(data) / max(data)
     if min_max_ratio < _eps:
         log.warning(
-            "min to max ratio is too low at %f and it could cause algorithm stability issues. Try to remove insignificant data",
-            min_max_ratio,
+            f"min to max ratio is too low at {min_max_ratio} and it could cause algorithm stability issues. "
+            "Try to remove insignificant data",
         )
     assert data == sorted(data, reverse=True), "data must be sorted (desc)"
     placed_circles = []
     radiuses = [math.sqrt(value) for value in data]
     for radius, next_ in look_ahead(radiuses):
-        placed_circles = place_new_A1_0(
-            radius, next_, placed_circles, get_hole_degree_radius_w
-        )
+        placed_circles = place_new_A1_0(radius, next_, placed_circles, get_hole_degree_radius_w)
     return placed_circles
 
 
 def extendBasis(B, p):
-    """Extend basis to ...  """
+    """Extend basis to ..."""
     if enclosesWeakAll(p, B):
         return [p]
 
@@ -506,7 +505,7 @@ def density(circles, enclosure=None):
     """
     if not enclosure:
         enclosure = enclose(circles)
-    return sum(c.r ** 2.0 for c in circles if c != enclosure) / enclosure.r ** 2.0
+    return sum(c.r**2.0 for c in circles if c != enclosure) / enclosure.r**2.0
 
 
 def look_ahead(iterable, n_elems=1):
@@ -521,7 +520,7 @@ def look_ahead(iterable, n_elems=1):
 
 
 def _handle(data, level, fields=None):
-    """Converts possibly heterogeneous list of float or dict in list of Output.
+    """Convert possibly heterogeneous list of float or dict in list of Output.
 
     Return:
         list of list of Output. There is one list per level and the (level
@@ -529,7 +528,7 @@ def _handle(data, level, fields=None):
 
     """
     if fields is None:
-        fields = FieldNames(None, None, None)
+        fields = Fields(None, None, None)
     datum_field = fields.datum if fields.datum else "datum"
     elements = []
     for datum in data:
@@ -541,13 +540,13 @@ def _handle(data, level, fields=None):
             raise ValueError("input data must be positive. Found " + str(datum))
         if datum <= _eps:
             log.warning(
-                "input data %f is small and could cause stability issues. Can you scale the data set up or drop insignificant elements?",
-                datum,
+                f"input data {datum} is small and could cause stability issues. "
+                "Can you scale the data set up or drop insignificant elements?"
             )
         try:
             elements.append(Circle(r=datum + 0, level=level, ex={"datum": datum}))
-        except TypeError:  # if it fails, assume dict.
-            raise TypeError("dict or numeric value expected")
+        except TypeError as err:  # if it fails, assume dict.
+            raise TypeError("dict or numeric value expected") from err
     return sorted(elements, reverse=True)
 
 
@@ -560,7 +559,7 @@ def _circlify_level(data, target_enclosure, fields, level=1):
     Args:
         elements (list of Circle): structured data to be process.
         target_enclosure (Circle): target enclosure to fit the cirlces into.
-        fields (FieldNames): field names.
+        fields (Fields): field names.
         level (int): level of depth in the hierarchy.
 
     Returns:
@@ -578,24 +577,15 @@ def _circlify_level(data, target_enclosure, fields, level=1):
         circle.level = level
         circle.circle = scale(inner_circle, target_enclosure, enclosure)
         if circle.ex and fields.children in circle.ex:
-            all_circles += _circlify_level(
-                circle.ex[fields.children], circle.circle, fields, level + 1
-            )
+            all_circles += _circlify_level(circle.ex[fields.children], circle.circle, fields, level + 1)
+        elif __debug__:
+            for key in circle.ex:
+                if key not in [fields.id, fields.datum, fields.children]:
+                    log.warning("unexpected key '%s' in input is ignored", key)
         all_circles.append(circle)
     return all_circles
 
 
-def _flatten(elements, flattened):
-    """Flattens the elements hierarchy."""
-    if elements is None:
-        return
-    for elem in elements:
-        _flatten(elem.children, flattened)
-        elem.children = None
-        flattened.append(elem)
-    return flattened
-
-
 def circlify(
     data,
     target_enclosure=None,
@@ -623,7 +613,7 @@ def circlify(
         (root to leaf) and descending value (biggest circles first).
 
     """
-    fields = FieldNames(id=id_field, datum=datum_field, children=children_field)
+    fields = Fields(id=id_field, datum=datum_field, children=children_field)
     if target_enclosure is None:
         target_enclosure = Circle(level=0, x=0.0, y=0.0, r=1.0)
     all_circles = _circlify_level(data, target_enclosure, fields)
